describe('ZhLayoutConfigController', function(){

    var zhPreferences;
    var zhLayoutPreferences;
    var zhLayoutWidgetsBuilder;
    var screenService;
    var messageService;
    var zhPromise;
    var zhLayoutCreateLayoutOptionsController;
    var ZhLayoutImportLayoutController;
    var $timeout;

    beforeEach(function(){
        zhPromise = Q;
        $timeout = jasmine.createSpy('$timeout').and.callFake(function(cbk){
            cbk();
        });

        zhPreferences                         = jasmine.createSpyObj('zhPreferences',                         ['getUserID', 'changeDefaultPreferences']);
        zhLayoutPreferences                   = jasmine.createSpyObj('zhLayoutPreferences',                   ['deleteLayouts', 'updateLayouts', 'applyLayout', 'downloadPreferenceFile']);
        zhLayoutWidgetsBuilder                = jasmine.createSpyObj('zhLayoutWidgetsBuilder',                ['factoryLayoutConfigurationWidget']);
        screenService                         = jasmine.createSpyObj('screenService',                         ['openPopup', 'closePopup', 'showMessage', 'i18n']);
        messageService                        = jasmine.createSpyObj('messageService',                        ['confirmMessageByCode', 'showMessageByCode', 'getMessageByCode']);
        zhLayoutCreateLayoutOptionsController = jasmine.createSpyObj('zhLayoutCreateLayoutOptionsController', ['getCreateLayoutOptionsWidget']);
        ZhLayoutImportLayoutController        = jasmine.createSpyObj('ZhLayoutImportLayoutController',        ['getImportLayoutWidget']);

        zhLayoutConfigController = new ZhLayoutConfigController(zhPreferences, zhLayoutPreferences, zhLayoutWidgetsBuilder, screenService, messageService, zhPromise, $timeout, zhLayoutCreateLayoutOptionsController, ZhLayoutImportLayoutController);
    });

    it('getConfigWidget', function(done){
        var userID = 1;

        var listWidget = {'id': 'listWidget'};
        var dataSource = {
            'name': 'DataSource',
            'data': [{
                'ID': 'row1'
            }, {
                'ID': 'ORIGINAL'
            }]
        };

        var configWidget = {
            'id': 'layoutConfigWidget'
        };

        zhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget.and.callFake(function(){
            return configWidget;
        });

        zhPreferences.getUserID.and.callFake(function(){
            return userID;
        });

        zhLayoutConfigController.getConfigWidget(listWidget, dataSource).then(function(factoredConfigWidget){
            expect(factoredConfigWidget.listWidget).toBe(listWidget);
            expect(factoredConfigWidget.dataSourceFilter).toEqual([
                {'name': 'WIDGET_ID', 'operator': '=', 'value': 'listWidget'}
            ]);
            expect(factoredConfigWidget.dataSource).toBe(dataSource);
            expect(dataSource.data[0].APPLIED).toBeFalsy();
            expect(dataSource.data[1].APPLIED).toBeTruthy();

            zhLayoutConfigController.getConfigWidget(listWidget, dataSource).then(function(configWidgetSecondCall){
                expect(configWidget).toBe(configWidgetSecondCall);
                done();
            });
        });
    });

    it('getConfigWidget: listWidget with appliedLayout', function(done){
        var userID = 1;

        var listWidget = {
            'id': 'listWidget',
            'name': 'list',
            'appliedLayout': {
                'ID': 'appliedLayoutID'
            }
        };
        var dataSource = {
            'name': 'DataSource',
            'data': [{
                'ID': 'appliedLayoutID'
            }, {
                'ID': 'ORIGINAL'
            }]
        };

        var configWidget = {
            'id': 'layoutConfigWidget'
        };

        zhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget.and.callFake(function(){
            return configWidget;
        });

        zhPreferences.getUserID.and.callFake(function(){
            return userID;
        });

        zhLayoutConfigController.getConfigWidget(listWidget, dataSource).then(function(factoredConfigWidget){
            expect(factoredConfigWidget.listWidget).toBe(listWidget);
            expect(factoredConfigWidget.dataSourceFilter).toEqual([
                {'name': 'WIDGET_ID', 'operator': '=', 'value': 'listWidget'}
            ]);
            expect(factoredConfigWidget.dataSource).toBe(dataSource);
            expect(dataSource.data[0].APPLIED).toBeTruthy();

            zhLayoutConfigController.getConfigWidget(listWidget, dataSource).then(function(configWidgetSecondCall){
                expect(configWidget).toBe(configWidgetSecondCall);
                done();
            });
        });
    });

    it('openCreateLayoutWidget', function(done){
        var container = {
            'id': 'container-1'
        };

        var listWidget = {
            'id': 'listWidget',
            'container': container
        };

        var listWidget2 = {
            'id': 'listWidget2',
            'container': container
        };

        container.widgets = [listWidget, listWidget2];

        var configWidget = {
            'id': 'layoutConfigWidget',
            'listWidget': listWidget
        };

        var createNewLayoutWidget = {
            'isValid': function(){return true;},
            'configWidget': configWidget,
            'currentRow': {
                'LABEL_TO_PREFERENCE': 'Layout Name'
            }
        };

        var userID = 'user-1';

        zhPreferences.getUserID.and.callFake(function(){
            return Q.when(userID);
        });

        zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget.and.callFake(function(){
            return Q.when(createNewLayoutWidget);
        });

        screenService.openPopup.and.callFake(function(){
            return Q.when();
        });

        zhLayoutConfigController.openCreateLayoutWidget(configWidget).then(function(){
            expect(createNewLayoutWidget.configWidget).toBe(configWidget);
            expect(screenService.openPopup).toHaveBeenCalledWith(createNewLayoutWidget);
            done();
        });
    });

    it('editLayout', function(done){
        var layout = {'id': 'layout-1'};

        var configWidget = {
            'listWidget': {'id': 1},
            'id': 'layoutConfigWidget'
        };

        var createLayoutOptionsWidget = {'id': 'createLayoutOptionsWidget'};

        zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget.and.callFake(function(){
            return Q.when(createLayoutOptionsWidget);
        });

        zhLayoutConfigController.editLayout(configWidget, layout).then(function(){
            expect(zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget).toHaveBeenCalledWith(layout, configWidget);
            expect(createLayoutOptionsWidget.createNewLayoutWidget).toEqual({'configWidget': configWidget});
            expect(screenService.openPopup).toHaveBeenCalledWith(createLayoutOptionsWidget);
            done();
        });
    });

    it('deleteLayouts', function(done){
        var layoutsToDelete = [{'ID': 1}, {'ID': 2}];

        var configWidget = {
            'listWidget': {'id': 1},
            'getCheckedRows': jasmine.createSpy('getCheckedRows').and.callFake(function(){
                return layoutsToDelete;
            }),
            'dataSource': {
                'name': 'dataSource',
                'clearCheckedRows': jasmine.createSpy('clearCheckedRows')
            }
        };

        messageService.confirmMessageByCode.and.callFake(function(){
            return Q.when();
        });

        zhLayoutPreferences.deleteLayouts.and.callFake(function(){
            return Q.when();
        });

        zhLayoutConfigController.deleteLayouts(configWidget).then(function(){
            expect(messageService.confirmMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#delete', {'NR_OF_LAYOUTS': 2});
            expect(zhLayoutPreferences.deleteLayouts).toHaveBeenCalledWith(layoutsToDelete);
            expect(configWidget.dataSource.clearCheckedRows).toHaveBeenCalled();
            expect(zhLayoutPreferences.updateLayouts).toHaveBeenCalledWith(1, configWidget.dataSource);
            done();
        });
    });

    it('deleteLayouts: error on delete default layout', function(){
        var layoutsToDelete = [{'ID': 1, 'DEFAULT_LAYOUT': true}, {'ID': 2}];
        var configWidget = {
            'listWidget': {
                'ID': '1'
            },
            'getCheckedRows': function(){},
            'dataSource': jasmine.createSpyObj('dataSource', ['clearCheckedRows'])
        };

        spyOn(configWidget, 'getCheckedRows').and.callFake(function(){
            return layoutsToDelete;
        });

        zhLayoutConfigController.deleteLayouts(configWidget);
        expect(messageService.showMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#delete_error_default');
        expect(zhLayoutPreferences.deleteLayouts).not.toHaveBeenCalled();
    });

    it('deleteLayouts: with error', function(done){
        var layoutsToDelete = [{'ID': 1}, {'ID': 2}];

        var configWidget = {
            'listWidget': {'id': 1},
            'getCheckedRows': jasmine.createSpy('getCheckedRows').and.callFake(function(){
                return layoutsToDelete;
            }),
            'dataSource': {
                'name': 'dataSource',
                'clearCheckedRows': jasmine.createSpy('clearCheckedRows')
            }
        };

        messageService.confirmMessageByCode.and.callFake(function(){
            return Q.when();
        });

        zhLayoutPreferences.deleteLayouts.and.callFake(function(){
            return Q.reject(new Error('error_code'));
        });

        screenService.i18n.and.callFake(function(word) {
            return 'translated ' + word;
        });

        messageService.getMessageByCode.and.callFake(function() {
            return Q.when('error message');
        });

        zhLayoutConfigController.deleteLayouts(configWidget).then(function(){
            expect(messageService.confirmMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#delete', {'NR_OF_LAYOUTS': 2});
            expect(zhLayoutPreferences.deleteLayouts).toHaveBeenCalledWith(layoutsToDelete);
            expect(configWidget.dataSource.clearCheckedRows).not.toHaveBeenCalled();
            expect(zhLayoutPreferences.updateLayouts).not.toHaveBeenCalled();
            expect(messageService.getMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#error_code');
            expect(messageService.showMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#delete_error', {'ERROR': 'translated error message'});
            expect(screenService.i18n).toHaveBeenCalledWith('error message');
            done();
        });
    });

    it('updateDefaultLayout', function(done){
        var configWidget = {
            'listWidget': {
                'id': 2
            }
        };

        var layout = {
            'ID': 5,
            'DEFAULT_LAYOUT': true
        };

        zhPreferences.changeDefaultPreferences.and.callFake(function(){
            return Q.when({'DEFAULT_LAYOUT': 5});
        });

        zhLayoutConfigController.updateDefaultLayout(configWidget, layout).then(function(response){
            expect(zhPreferences.changeDefaultPreferences).toHaveBeenCalledWith(configWidget, 'DEFAULT_LAYOUT', layout);
            expect(response).toEqual({'DEFAULT_LAYOUT': 5});
            done();
        });
    });

    xit('updateDefaultLayout: error', function(done){
        var configWidget = {
            'listWidget': {
                'id': 2
            }
        };

        var layout = {
            'ID': 5,
            'DEFAULT_LAYOUT': false
        };

        zhPreferences.changeDefaultPreferences.and.callFake(function(){
            return Q.reject(new Error('update default layout error'));
        });

        zhLayoutConfigController.updateDefaultLayout(configWidget, layout).then(function(response){
            expect(zhPreferences.changeDefaultPreferences).toHaveBeenCalledWith(2, null);
            expect(screenService.showMessage).toHaveBeenCalledWith('update default layout error');
            done();
        });
    });

    it('updateAppliedLayout: apply a layout', function(done){
        var configWidget = {
            'listWidget': {
                'id': 2
            },
            'dataSource': {
                'data': [
                    {'ID': 1, 'DEFAULT_LAYOUT': false},
                    {'ID': 2, 'DEFAULT_LAYOUT': false},
                    {'ID': 3, 'DEFAULT_LAYOUT': false},
                    {'ID': 4, 'DEFAULT_LAYOUT': false},
                    {'ID': 5, 'DEFAULT_LAYOUT': false}
                ]
            }
        };

        var layout = {
            'ID': 5,
            'DEFAULT_LAYOUT': true
        };

        zhLayoutPreferences.applyLayout.and.callFake(function(){
            return Q.when();
        });

        zhLayoutConfigController.updateAppliedLayout(configWidget, layout).then(function(){
            expect(zhLayoutPreferences.applyLayout).toHaveBeenCalledWith(configWidget.listWidget, layout);
            expect(configWidget.dataSource.data).toEqual([
                {'ID': 1, 'DEFAULT_LAYOUT': false, 'APPLIED': false},
                {'ID': 2, 'DEFAULT_LAYOUT': false, 'APPLIED': false},
                {'ID': 3, 'DEFAULT_LAYOUT': false, 'APPLIED': false},
                {'ID': 4, 'DEFAULT_LAYOUT': false, 'APPLIED': false},
                {'ID': 5, 'DEFAULT_LAYOUT': false, 'APPLIED': true}
            ]);
            done();
        });
    });

    it('downloadPreferenceFile', function(done){
        zhLayoutPreferences.downloadPreferenceFile.and.callFake(function(){
            return Q.when();
        });

        var layout = {
            'ID': 5,
            'DEFAULT_LAYOUT': true
        };

        zhLayoutConfigController.downloadPreferenceFile(layout).then(function(){
            expect(zhLayoutPreferences.downloadPreferenceFile).toHaveBeenCalled();
            done();
        });

    });

    it('importPreferenceFile ', function(done){
        var configWidget = {
            'listWidget': {
                'id': 2
            },
            'dataSource': {
                'data': [{
                    'ID': 5
                }]
            }
        };

        var importLayoutWidget = {
            'name': 'zhLayoutImportLayoutWidgetController',
            'newRow': function(){}
        };

        spyOn(importLayoutWidget, 'newRow');

        ZhLayoutImportLayoutController.getImportLayoutWidget.and.callFake(function(){
            return Q.when(importLayoutWidget);
        });

        screenService.openPopup.and.callFake(function(){
            return Q.when('{}');
        });

        zhLayoutConfigController.importPreferenceFile(configWidget).then(function(){
            expect(importLayoutWidget.newRow).toHaveBeenCalled();
            expect(screenService.openPopup).toHaveBeenCalledWith(importLayoutWidget);
            done();
        });

    });

});